using Syntriax.Engine.Core;
using Syntriax.Engine.Physics2D;

namespace Pong.Behaviours;

public class MovementBallBehaviour : Behaviour2D, IFirstFrameUpdate, IUpdate
{
    public Vector2D StartDirection { get; private set; } = Vector2D.Zero;
    public float Speed { get; set; } = 500f;
    public float SpeedUpMultiplier { get; set; } = .0125f;

    private IRigidBody2D rigidBody = null!;

    public void FirstActiveFrame()
    {
        rigidBody = BehaviourController.GetRequiredBehaviour<IRigidBody2D>();

        rigidBody.Velocity = StartDirection * Speed;
        BehaviourController.GetRequiredBehaviour<ICollider2D>().OnCollisionDetected.AddListener(OnCollisionDetected);
    }

    public void Update()
    {
        if (rigidBody.Velocity.MagnitudeSquared <= 0.01f)
            return;

        Vector2D speedUp = rigidBody.Velocity.Normalized * Universe.Time.DeltaTime;
        rigidBody.Velocity += speedUp * SpeedUpMultiplier;
    }

    private void OnCollisionDetected(ICollider2D collider2D, CollisionDetectionInformation information)
    {
        if (Syntriax.Engine.Core.Math.Abs(information.Normal.Dot(Vector2D.Right)) > .25)
            rigidBody.Velocity = information.Detector.Transform.Position.FromTo(information.Detected.Transform.Position).Normalized * rigidBody.Velocity.Magnitude;
        else
            rigidBody.Velocity = rigidBody.Velocity.Reflect(information.Normal);
    }

    public MovementBallBehaviour() { }
    public MovementBallBehaviour(Vector2D startDirection, float speed)
    {
        StartDirection = Vector2D.Normalize(startDirection);
        Speed = speed;
    }
}
